@@ -20,7 +20,6 @@ namespace amici {
 
 SteadystateProblem::SteadystateProblem(const Solver &solver, const Model &model)
     : delta_(model.nx_solver), ewt_(model.nx_solver), ewtQB_(model.nplist()),
-      rel_x_newton_(model.nx_solver), x_newton_(model.nx_solver),
       x_(model.nx_solver), x_old_(model.nx_solver), dx_(model.nx_solver),
       xdot_(model.nx_solver), xdot_old_(model.nx_solver),
       sx_(model.nx_solver, model.nplist()), sdx_(model.nx_solver, model.nplist()),
@@ -284,7 +283,7 @@ void SteadystateProblem::getQuadratureByLinSolve(NewtonSolver *newtonSolver,
         /* compute integral over xB and write to xQ */
         newtonSolver->prepareLinearSystemB(0, -1);
         newtonSolver->solveLinearSystem(xQ_);
-        /* Compute the quadrature as the inner product xQ * dxotdp */
+        /* Compute the quadrature as the inner product xQ * dxdotdp */
         computeQBfromQ(model, xQ_, xQB_);
         /* set flag that quadratures is available (for processing in rdata) */
         hasQuadrature_ = true;
@@ -505,11 +504,10 @@ void SteadystateProblem::applyNewtonsMethod(Model *model,
 
     /* Check for relative error, but make sure not to divide by 0!
         Ensure positivity of the state */
-    x_newton_ = x_;
     x_old_ = x_;
     xdot_old_ = xdot_;
 
-    wrms_ = getWrmsNorm(x_newton_, xdot_, newtonSolver->atol_,
+    wrms_ = getWrmsNorm(x_, xdot_, newtonSolver->atol_,
                         newtonSolver->rtol_, ewt_);
     bool converged = newton_retry ? false : wrms_ < RCONST(1.0);
     while (!converged && i_newtonstep < newtonSolver->max_steps) {
@@ -530,7 +528,7 @@ void SteadystateProblem::applyNewtonsMethod(Model *model,
 
         /* Compute new xdot and residuals */
         model->fxdot(t_, x_, dx_, xdot_);
-        realtype wrms_tmp = getWrmsNorm(x_newton_, xdot_, newtonSolver->atol_,
+        realtype wrms_tmp = getWrmsNorm(x_, xdot_, newtonSolver->atol_,
                                         newtonSolver->rtol_, ewt_);
 
         if (wrms_tmp < wrms_) {
@@ -555,7 +553,7 @@ void SteadystateProblem::applyNewtonsMethod(Model *model,
                 }
                 if (recheck_convergence) {
                   model->fxdot(t_, x_, dx_, xdot_);
-                  wrms_ = getWrmsNorm(x_newton_, xdot_, newtonSolver->atol_,
+                  wrms_ = getWrmsNorm(x_, xdot_, newtonSolver->atol_,
                                      newtonSolver->rtol_, ewt_);
                   converged = wrms_ < RCONST(1.0);
                 }
@@ -611,6 +609,7 @@ void SteadystateProblem::runSteadystateSimulation(const Solver *solver,
     if (backward)
         sensitivityFlag = SensitivityMethod::adjoint;
 
+    /* If run after Newton's method checks again if it converged */
     bool converged = checkConvergence(solver, model, sensitivityFlag);
     int sim_steps = 0;
 
@@ -695,7 +694,7 @@ std::unique_ptr<Solver> SteadystateProblem::createSteadystateSimSolver(
 
 void SteadystateProblem::computeQBfromQ(Model *model, const AmiVector &yQ,
                                         AmiVector &yQB) const {
-    /* Compute the quadrature as the inner product: yQB = dxotdp * yQ */
+    /* Compute the quadrature as the inner product: yQB = dxdotdp * yQ */
 
     /* set to zero first, as multiplication adds to existing value */
     yQB.zero();
